@@ -90,6 +90,25 @@ void EventQueue::SetFilter(std::unique_ptr<Expression> filter)
 	m_Filter.swap(filter);
 }
 
+Dictionary::Ptr EventQueue::WaitForEvent(void *client, double timeout)
+{
+	boost::mutex::scoped_lock lock(m_Mutex);
+
+	for (;;) {
+		auto it = m_Events.find(client);
+		ASSERT(it != m_Events.end());
+
+		if (!it->second.empty()) {
+			Dictionary::Ptr result = *it->second.begin();
+			it->second.pop_front();
+			return result;
+		}
+
+		if (!m_CV.timed_wait(lock, boost::posix_time::milliseconds(long(timeout * 1000))))
+			return nullptr;
+	}
+}
+
 std::vector<EventQueue::Ptr> EventQueue::GetQueuesForType(const String& type)
 {
 	EventQueueRegistry::ItemMap queues = EventQueueRegistry::GetInstance()->GetItems();
@@ -126,7 +145,7 @@ EventQueueRegistry *EventQueueRegistry::GetInstance()
 }
 
 std::mutex EventsInbox::m_FiltersMutex;
-std::map<String, EventsInbox::Filter> EventsInbox::m_Filters ({{"", EventsInbox::Filter{1, nullptr}}});
+std::map<String, EventsInbox::Filter> EventsInbox::m_Filters ({{"", EventsInbox::Filter{1, Expression::Ptr()}}});
 
 EventsRouter EventsRouter::m_Instance;
 
@@ -146,7 +165,7 @@ EventsInbox::EventsInbox(String filter, const String& filterSource)
 		m_Filter = m_Filters.find(filter);
 
 		if (m_Filter == m_Filters.end()) {
-			m_Filter = m_Filters.emplace(std::move(filter), Filter{1, std::shared_ptr<Expression>(expr.release())}).first;
+			m_Filter = m_Filters.emplace(std::move(filter), Filter{1, Expression::Ptr(expr.release())}).first;
 		} else {
 			++m_Filter->second.Refs;
 		}
@@ -164,7 +183,7 @@ EventsInbox::~EventsInbox()
 	}
 }
 
-const std::shared_ptr<Expression>& EventsInbox::GetFilter()
+const Expression::Ptr& EventsInbox::GetFilter()
 {
 	return m_Filter->second.Expr;
 }
@@ -230,7 +249,7 @@ const EventsInbox::Ptr& EventsSubscriber::GetInbox()
 	return m_Inbox;
 }
 
-EventsFilter::EventsFilter(std::map<std::shared_ptr<Expression>, std::set<EventsInbox::Ptr>> inboxes)
+EventsFilter::EventsFilter(std::map<Expression::Ptr, std::set<EventsInbox::Ptr>> inboxes)
 	: m_Inboxes(std::move(inboxes))
 {
 }
